@@ -124,6 +124,13 @@ void MessagesModel::setupChannelSignals(const Tp::TextChannelPtr &channel)
     connect(channel.data(),
             SIGNAL(pendingMessageRemoved(Tp::ReceivedMessage)),
             SLOT(onPendingMessageRemoved()));
+
+    connect(channel.data(), &Tp::TextChannel::messageReceived,
+            this, &MessagesModel::lastMessageChanged);
+    connect(channel.data(), &Tp::TextChannel::messageSent,
+            this, &MessagesModel::lastMessageChanged);
+    connect(channel.data(), &Tp::TextChannel::pendingMessageRemoved,
+            this, &MessagesModel::lastMessageChanged);
 }
 
 void MessagesModel::setTextChannel(const Tp::TextChannelPtr &channel)
@@ -161,6 +168,16 @@ void MessagesModel::setTextChannel(const Tp::TextChannelPtr &channel)
     }
 }
 
+Tp::AccountPtr MessagesModel::account() const
+{
+    return d->account;
+}
+
+void MessagesModel::setAccount(const Tp::AccountPtr &account)
+{
+    d->account = account;
+}
+
 void MessagesModel::onHistoryFetched(const QList<KTp::Message> &messages)
 {
     if (!messages.isEmpty()) {
@@ -172,6 +189,8 @@ void MessagesModel::onHistoryFetched(const QList<KTp::Message> &messages)
         endInsertRows();
     }
     d->logsLoaded = true;
+
+    Q_EMIT lastMessageChanged();
 }
 
 void MessagesModel::onMessageReceived(const Tp::ReceivedMessage &message)
@@ -235,6 +254,14 @@ void MessagesModel::onMessageReceived(const Tp::ReceivedMessage &message)
 
         endInsertRows();
 
+        // Update the previous message in the view
+        // This will redraw the part of the bubble to not
+        // be bottom but middle one, if this is a consecutive
+        // message to the previous one
+        if (d->messages.count() > 1) {
+            Q_EMIT dataChanged(createIndex(newMessageIndex - 1, 0), createIndex(newMessageIndex - 1, 0));
+        }
+
         if (d->visible) {
             acknowledgeAllMessages();
         } else {
@@ -247,8 +274,8 @@ void MessagesModel::onMessageSent(const Tp::Message &message, Tp::MessageSending
 {
     Q_UNUSED(flags);
 
-    int length = rowCount();
-    beginInsertRows(QModelIndex(), length, length);
+    int newMessageIndex = rowCount();
+    beginInsertRows(QModelIndex(), newMessageIndex, newMessageIndex);
 
     const KTp::Message &newMessage = KTp::MessageProcessor::instance()->processIncomingMessage(
                 message, d->account, d->textChannel);
@@ -256,11 +283,19 @@ void MessagesModel::onMessageSent(const Tp::Message &message, Tp::MessageSending
 
     if (!messageToken.isEmpty()) {
         // Insert the message into the lookup table for delivery reports.
-        const QPersistentModelIndex modelIndex = createIndex(length, 0);
+        const QPersistentModelIndex modelIndex = createIndex(newMessageIndex, 0);
         d->messagesByMessageToken.insert(messageToken, modelIndex);
     }
 
     endInsertRows();
+
+    // Update the previous message in the view
+    // This will redraw the part of the bubble to not
+    // be bottom but middle one, if this is a consecutive
+    // message to the previous one
+    if (d->messages.count() > 1) {
+        Q_EMIT dataChanged(createIndex(newMessageIndex - 1, 0), createIndex(newMessageIndex - 1, 0));
+    }
 }
 
 void MessagesModel::onPendingMessageRemoved()
@@ -425,7 +460,7 @@ QString MessagesModel::lastMessage() const
         return QString();
     }
 
-    return data(index, MessagesModel::TextRole).toString();
+    return data(index, MessagesModel::TextRole).toString().simplified();
 }
 
 QDateTime MessagesModel::lastMessageDateTime() const
